@@ -1,5 +1,5 @@
 __author__ = 'Matteo Cafasso'
-__version__ = '5.0.7'
+__version__ = '5.1.0'
 __license__ = 'LGPL'
 
 
@@ -15,7 +15,8 @@ __all__ = ['waitforthreads',
            'ThreadPool']
 
 
+from pebble import concurrent, asynchronous
 from pebble.decorators import synchronized, sighandler
-from pebble.common import ProcessExpired, ProcessFuture
 from pebble.functions import waitforqueues, waitforthreads
+from pebble.common import ProcessExpired, ProcessFuture, CONSTS
 from pebble.pool import ThreadPool, ProcessPool, MapFuture, ProcessMapFuture
