@@ -13,7 +13,7 @@ from concurrent.futures import CancelledError, TimeoutError
 
 import pebble
 from pebble import ProcessPool, ProcessExpired
-from pebble.pool.base_pool import ERROR
+from pebble.pool.base_pool import PoolStatus
 
 
 # set start method
@@ -427,7 +427,11 @@ class TestProcessPool(unittest.TestCase):
         """Process Pool Fork unexpect death of worker raises ProcessExpired."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(suicide_function)
-            self.assertRaises(ProcessExpired, future.result)
+            worker_pid = list(pool._pool_manager.worker_manager.workers)[0]
+            with self.assertRaises(ProcessExpired) as exc_ctx:
+                future.result()
+            self.assertEqual(exc_ctx.exception.exitcode, 1)
+            self.assertEqual(exc_ctx.exception.pid, worker_pid)
 
     def test_process_pool_map(self):
         """Process Pool Fork map simple."""
@@ -549,7 +553,7 @@ class TestProcessPool(unittest.TestCase):
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.map(long_function, elements, timeout=1)
             generator = future.result()
-            pool._context.state = ERROR
+            pool._context.status = PoolStatus.ERROR
             while True:
                 try:
                     next(generator)
