@@ -6,13 +6,14 @@ import signal
 import asyncio
 import unittest
 import threading
+import dataclasses
 import multiprocessing
 
 from concurrent.futures import CancelledError, TimeoutError
 
 import pebble
 from pebble import ProcessPool, ProcessExpired
-from pebble.pool.base_pool import ERROR
+from pebble.pool.base_pool import PoolStatus
 
 
 # set start method
@@ -28,7 +29,7 @@ if 'forkserver' in methods:
         if mp_context.get_start_method() == 'forkserver':
             supported = True
         else:
-            raise Exception(mp_context.get_start_method())
+            raise BaseException(mp_context.get_start_method())
     except RuntimeError:  # child process
         pass
 
@@ -46,7 +47,7 @@ def long_initializer():
 
 
 def broken_initializer():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
 
 
 def function(argument, keyword_argument=0):
@@ -59,13 +60,26 @@ def initializer_function():
 
 
 def error_function():
-    raise Exception("BOOM!")
+    raise BaseException("BOOM!")
+
+
+def return_error_function():
+    return BaseException("BOOM!")
 
 
 def pickle_error_function():
     return threading.Lock()
 
 
+@dataclasses.dataclass(frozen=True)
+class FrozenError(Exception):
+    pass
+
+
+def frozen_error_function():
+    raise FrozenError()
+
+
 def long_function(value=1):
     time.sleep(value)
     return value
@@ -122,7 +136,7 @@ class TestProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        except Exception as error:
+        except BaseException as error:
             self.exception = error
         finally:
             self.event.set()
@@ -155,7 +169,13 @@ class TestProcessPool(unittest.TestCase):
         """Process Pool Forkserver errors are raised by future get."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
-        self.assertRaises(Exception, future.result)
+        self.assertRaises(BaseException, future.result)
+
+    def test_process_pool_error_returned(self):
+        """Process Pool Forkserver returned errors are returned by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(return_error_function)
+        self.assertIsInstance(future.result(), BaseException)
 
     def test_process_pool_error_callback(self):
         """Process Pool Forkserver errors are forwarded to callback."""
@@ -163,7 +183,7 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(error_function)
         future.add_done_callback(self.callback)
         self.event.wait()
-        self.assertTrue(isinstance(self.exception, Exception))
+        self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_pickling_error_task(self):
         """Process Pool Forkserver task pickling errors
@@ -179,6 +199,12 @@ class TestProcessPool(unittest.TestCase):
             future = pool.schedule(pickle_error_function)
             self.assertRaises((pickle.PicklingError, TypeError), future.result)
 
+    def test_process_pool_frozen_error(self):
+        """Process Pool Forkserver frozen errors are raised by future get."""
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            future = pool.schedule(frozen_error_function)
+        self.assertRaises(FrozenError, future.result)
+
     def test_process_pool_timeout(self):
         """Process Pool Forkserver future raises TimeoutError if so."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
@@ -380,12 +406,12 @@ class TestProcessPool(unittest.TestCase):
                           kwargs={'keyword_argument': 1})
 
     def test_process_pool_exception_isolated(self):
-        """Process Pool Forkserver an Exception does not affect other futures."""
+        """Process Pool Forkserver an BaseException does not affect other futures."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(error_function)
             try:
                 future.result()
-            except Exception:
+            except BaseException:
                 pass
             future = pool.schedule(function, args=[1],
                                    kwargs={'keyword_argument': 1})
@@ -403,7 +429,11 @@ class TestProcessPool(unittest.TestCase):
         """Process Pool Forkserver unexpect death of worker raises ProcessExpired."""
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.schedule(suicide_function)
-            self.assertRaises(ProcessExpired, future.result)
+            worker_pid = list(pool._pool_manager.worker_manager.workers)[0]
+            with self.assertRaises(ProcessExpired) as exc_ctx:
+                future.result()
+            self.assertEqual(exc_ctx.exception.exitcode, 1)
+            self.assertEqual(exc_ctx.exception.pid, worker_pid)
 
     def test_process_pool_map(self):
         """Process Pool Forkserver map simple."""
@@ -524,7 +554,7 @@ class TestProcessPool(unittest.TestCase):
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             future = pool.map(long_function, elements, timeout=1)
             generator = future.result()
-            pool._context.state = ERROR
+            pool._context.status = PoolStatus.ERROR
             while True:
                 try:
                     next(generator)
@@ -564,7 +594,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
     def callback(self, future):
         try:
             self.result = future.result()
-        # asyncio.exception.CancelledError does not inherit from Exception
+        # asyncio.exception.CancelledError does not inherit from BaseException
         except BaseException as error:
             self.exception = error
         finally:
@@ -620,9 +650,19 @@ class TestAsyncIOProcessPool(unittest.TestCase):
             return await loop.run_in_executor(pool, error_function, None)
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
-            with self.assertRaises(Exception):
+            with self.assertRaises(BaseException):
                 asyncio.run(test(pool))
 
+    def test_process_pool_error_returned(self):
+        """Process Pool Forkserver returned errors are returned by future get."""
+        async def test(pool):
+            loop = asyncio.get_running_loop()
+
+            return await loop.run_in_executor(pool, return_error_function, None)
+
+        with ProcessPool(max_workers=1, context=mp_context) as pool:
+            self.assertIsInstance(asyncio.run(test(pool)), BaseException)
+
     def test_process_pool_error_callback(self):
         """Process Pool Forkserver errors are forwarded to callback."""
         async def test(pool):
@@ -638,7 +678,7 @@ class TestAsyncIOProcessPool(unittest.TestCase):
 
         with ProcessPool(max_workers=1, context=mp_context) as pool:
             asyncio.run(test(pool))
-            self.assertTrue(isinstance(self.exception, Exception))
+            self.assertTrue(isinstance(self.exception, BaseException))
 
     def test_process_pool_timeout(self):
         """Process Pool Forkserver future raises TimeoutError if so."""
